/*****************************************************************************
 *
 *  AngleFormatter.h                            XMM Science Analysis Subsystem
 *
 *  Angle with methods for parsing and formatting
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: AngleFormatter.h,v 1.3 2001/03/28 19:20:34 jbakker Exp $
 *
 *****************************************************************************/

#ifndef ANGLEFORMATTER_H
#define ANGLEFORMATTER_H

#include <string>              // C++ headers
#include <strstream>


//================================================================
//  Class AngleFormatter
//
//  AngleFormatter handles parsing and printing of angles in a
//  variety of formats. e.g:
//
//      12h34m56.789s    (HMS)
//      12H34M56.789S
//      12d34m56.789s    (DMS)
//     -12d34m56.789s
//      123.456          (Degrees)
//     -1.23456e2
//      12:34:56.789     (Colon separated DMS or HMS)
//
//================================================================

class AngleFormatter
{
public:
    enum AngleMode { Degrees, DMS, HMS };

    // Constructors
    AngleFormatter();
    AngleFormatter(AngleMode mode);

    // Format the angle as a string
    string asString(double value, char separator='\0') const;

    // Parse the string, return the value and set the mode
    bool parse(const string& str, double& value, char separator='\0');

    void setMode(AngleMode mode=Degrees);
    AngleMode mode() { return _mode; }

private:

    // Parsing 
    static bool parseFields(const string& text, char delim1, char delim2,
			    char delim3, double& result, AngleMode mode);

    static bool parseFields(const string& text, char separator,
			    double& result, AngleMode mode);

    static bool convertFields(int hrsDeg, int mins, double secs,
			      bool hoursMode, double& result);

    // Member data
    AngleMode  _mode;
};

#endif
