/*****************************************************************************
 *
 *  AngleParameter.cc                           XMM Science Analysis Subsystem
 *
 *  Angle task parameter
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: AngleParameter.cc,v 1.6 2001/03/28 19:20:34 jbakker Exp $
 *
 *****************************************************************************/

#include "AngleParameter.h"
#include "StringToken.h"
#include <errstr.h>

#include <iostream>


//================================================================
//  Class AngleParameter
//================================================================

/*** Constructor ***/

AngleParameter::AngleParameter()
{
    _mode = AngleFormatter::Degrees;
}


/*** Set the format mode ***/

void AngleParameter::setMode(AngleFormatter::AngleMode mode)
{
    _mode = mode;
    notifyView();
}


/*** Format the parameter value as a string ***/

string AngleParameter::asString() const
{
    AngleFormatter t(_mode);

    return "'" + t.asString(_value) + "'";
}


/*** Format the default value as a string ***/

string AngleParameter::defaultAsString() const
{
    AngleFormatter t(_mode);

    return "'" + t.asString(_default) + "'";
}


/***  Parse string representation and assign to 'value' if it is valid ***/

bool AngleParameter::parse(const string& str, double& value)
{
    StringToken token(str);
    AngleFormatter t;
    bool ok = t.parse(token.asUnquotedString(), value);

    if(ok) {
        _mode = t.mode();
    } else {
      errstr << "Parameter '" << _name << "'" << Sas::error("ParamWrongType");
    }

    return ok;
}

