/*****************************************************************************
 *
 *  IntParameter.h                              XMM Science Analysis Subsystem
 *
 *  Integer task parameter
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: IntParameter.h,v 1.9 2001/03/28 19:20:35 jbakker Exp $
 *
 *****************************************************************************/

#ifndef INTPARAMETER_H
#define INTPARAMETER_H

#include "Optional.h"            // SAS headers
#include "Parameter.h"

#include <string>                // C++ headers
#include <iostream>


//================================================================
//  Class IntParameter
//================================================================

class IntParameter : public Parameter
{
public:

    // Construction
    IntParameter* clone() { return new IntParameter(*this); }

    // Value
    int         intValue() const { return _value; }
    bool        setNewValue(const string& value);
    bool        setValue(int value);
    ParamType   getType() const {return Parameter::Int;}
    const char* getTypeName() const { return "int"; }

    // Default
    int        defaultValue() const {return _default;}
    bool       setNewDefault(const string& value);
    bool       setDefault(int value);
    void       revertToDefault();

    // Constraint
    void       setMin(int minVal) {_min = minVal;}
    void       setMax(int maxVal) {_max = maxVal;}
    const Optional<int>& minValue() const {return _min;}
    const Optional<int>& maxValue() const {return _max;}

    // Formatting
    string     asString() const;
    string     defaultAsString() const;

    // Validation
    bool       validate() const;

private:
    // called by Parameter::setValue(const Parameter&);
    bool assignValueOf(const Parameter& p) {return setValue(p.intValue());}


    // Parsing
    bool parse(const string& str, int& value);

    // Validation
    bool isValid(int n) const;
    void reportRangeError(const string& valueString);
    void writeConstraint(ostream& os) const;

    // Member data
    int            _value;
    int            _default;
    Optional<int>  _min;
    Optional<int>  _max;
};

#endif
