/*****************************************************************************
 *
 *  ParamInfoSet.cc                             XMM Science Analysis Subsystem
 *
 *  ParameterInfo for a set of tasks.
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: ParamInfoSet.cc,v 1.4 2001/03/28 19:20:35 jbakker Exp $
 *
 *****************************************************************************/

#include "ParamInfoSet.h"            // Application headers
#include "ParameterInfo.h"
#include "ParamInfoReader.h"
#include "XpiInfoReader.h"
#include <errstr.h>

#include <iostream>             // C++ headers


//================================================================
//  Class ParamInfoSet [Singleton]
//================================================================

ParamInfoSet* ParamInfoSet::_infoSet = 0;


/*** Return the single instance [STATIC] ***/

ParamInfoSet* ParamInfoSet::instance()
{
    if(!_infoSet)
        _infoSet = new ParamInfoSet();

    return _infoSet;
}


/*** Revert to defaults for all tasks (simply clears cache) ***/

void ParamInfoSet::resetParamInfo()
{
  errstr << "Unloading all parameters for all tasks in memory"
	 << Sas::message(Msg::Lib, Msg::Noisy);

    ParamInfoMap::const_iterator it = _infoMap.begin();
    while(it != _infoMap.end())
    {
        ParameterInfo* p = (*it++).second;
	delete p;
    }
    _infoMap.clear();    
}


/*** Revert to defaults for specified task ***/

void ParamInfoSet::resetParamInfo(const string& taskName)
{
  errstr << "Unloading all parameters for task (" << taskName << ")"
	 << Sas::message(Msg::Lib, Msg::Noisy);

    ParamInfoMap::iterator it = _infoMap.find(taskName);
    if(it != _infoMap.end())
    {
        ParameterInfo* p = (*it).second;
	delete p;
	_infoMap.erase(it);
    }
}


/*** Get the parameter info for a specified task ***/

ParameterInfo* ParamInfoSet::getParamInfo(const string& taskName)
{
    ParameterInfo* info = 0;

    if(_infoMap.find(taskName) == _infoMap.end())
    {
        errstr << "Reading parameters from file for task(" << taskName << ")"
	       << Sas::message(Msg::Lib, Msg::Noisy);

        ParamInfoReader* reader = new XpiInfoReader(taskName);
	if(reader)
	{
	    info = reader->getParamInfo();
	    _infoMap[taskName] = info;
	
	    delete reader;
	}
    } else {
        errstr << "Reading cached parameters for task(" << taskName << ")"
	       << Sas::message(Msg::Lib, Msg::Noisy);

        info = _infoMap[taskName];     // Return cached value
    }

    return info;
}


