/*****************************************************************************
 *
 *  ParamInfoSet.h                              XMM Science Analysis Subsystem
 *
 *  ParameterInfo for a set of tasks.
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: ParamInfoSet.h,v 1.2 2001/03/28 19:20:35 jbakker Exp $
 *
 *****************************************************************************/

#ifndef PARAMINFOSET_H
#define PARAMINFOSET_H

#include "Parameter.h"            // Application headers
#include "ParameterInfo.h"

#include <string>                 // C++ headers
#include <map>


//================================================================
//  Class ParamInfoSet [SINGLETON]
//
//  There is a single instance of this class which caches the
//  parameters associated with a task.
//================================================================

class ParamInfoSet
{
public:

    // Type definitions
    typedef map<string, ParameterInfo*, less<string> > ParamInfoMap;

    // Return the sole instance of this class
    static ParamInfoSet* instance();

    // Return the ParameterInfo for the specified task
    ParameterInfo* getParamInfo(const string& taskName);

    // Reset all parameters of all tasks to their default values
    void resetParamInfo();

    // Reset parameters of specified task to their default values
    void resetParamInfo(const string& taskName);

private:
    ParamInfoMap _infoMap;          // Maps task name onto ParameterInfo 

    static ParamInfoSet* _infoSet;  // The sole instance of this class
};

#endif
