/*****************************************************************************
 *
 *  StringListParameter.h                       XMM Science Analysis Subsystem
 *
 *  String-list task parameter
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: StringListParameter.h,v 1.10 2001/03/28 19:20:37 jbakker Exp $
 *
 *****************************************************************************/

#ifndef STRINGLISTPARAMETER_H
#define STRINGLISTPARAMETER_H

#include "Optional.h"            // SAS headers
#include "Parameter.h"

#include <string>                // C++ headers
#include <vector>
#include <list>


//================================================================
//  Class StringListParameter
//================================================================

class StringListParameter : public Parameter
{
public:
    // Construction
    StringListParameter* clone() { return new StringListParameter(*this); }

    // Value
    vector<string> stringListValue() const {return _value;}
    bool           setNewValue(const string& value);
    bool           setValue(const vector<string>& value);
    int            size() const;
    ParamType      getType() const {return Parameter::StringList;}
    const char*    getTypeName() const { return "string_list"; }

    // Default
    vector<string> defaultValue() const {return _default;}
    bool           setNewDefault(const string& value);
    bool           setDefault(const vector<string>& value);
    void           revertToDefault();

    // Constraint
    void setChoices(const list<string>& choices) { _choices = choices; }
    const list<string>& getChoices() const {return _choices;}

    // Fomatting
    string         asString() const;
    string         defaultAsString() const;

    // Validation
    virtual bool   validate() const;

protected:

    // Parsing
    bool parse(const string& str, vector<string>& value);

    // Validation
    virtual bool isValid(const vector<string>& n) const;
    virtual void reportRangeError(const string& valueString);
    virtual void writeConstraint(ostream&) const;

    // Member data
    vector<string> _value;
    vector<string> _default;

private:
    // called by Parameter::setValue(const Parameter&);
    bool assignValueOf(const Parameter& p) {return setValue(p.stringListValue());}

    list<string> splitStringList(const string& str);

    // Member data
    list<string> _choices;
};

#endif

