/*****************************************************************************
 *
 *  paramcc.cc                                  XMM Science Analysis Subsystem
 *
 *  C task interface to parameter system
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: paramcc.cc,v 1.9 2001/09/06 10:23:39 jbakker Exp $
 *
 *****************************************************************************/

#include "param.h"           // Application headers
#include "ParamInfoSet.h"

#include <iostream>        // C++ headers


int parameterCount(const string& name)
{
  int n = (*gParameters)[name]->size();
  /*
  if(n < 0)
    errstr << "Parameter '" << name << "' is incorrectly accessed as a list"
	   << Sas::error("ParamNotList");
  */
  return n;
}


/*** Return value of boolean parameter "name" ***/

bool booleanParameter(const string& name, int pos=-1)
{ 
  bool b = false;
  
  Parameter* p = (*gParameters)[name];
  if(p) {
    if (p->size() < 0 && pos > -1) {
      errstr << "Parameter '"
	     << name << "' is incorrectly accessed as a list"
	       << Sas::error("ParamNotList");      
    }
    if(pos < 0)
      b = p->boolValue();
    else if(pos < p->size())
      b = p->boolListValue()[pos];
    else
      errstr << "Index " << pos << " is out of range "
	     << "[" << 0 << ',' << p->size() << ") for list parameter '" 
	     << name << "'"
	     << Sas::error("ParamIndexRange");
  }
  return b;
}


vector<int> boolListParameter(const string& name)
{
  return (*gParameters)[name]->boolListValue();
}


/*** Return value of integer parameter "name" ***/

int intParameter(const string& name, int pos=-1)
{
  int n = 0;
  
  Parameter* p = (*gParameters)[name];
  
  if(p) {
    if (p->size() < 0 && pos > -1) {
      errstr << "Parameter '"
	     << name << "' is incorrectly accessed as a list"
	     << Sas::error("ParamNotList");      
    }
    
    if(pos < 0)
      n = p->intValue();
    else if(pos < p->size())
      n = p->intListValue()[pos];
    else
      errstr << "Index " << pos << " is out of range "
	     << "[" << 0 << ',' << p->size() << ") for list parameter '" 
	     << name << "'"
	     << Sas::error("ParamIndexRange");
  }
  return n;
}

vector<int> intListParameter(const string& name)
{
    return (*gParameters)[name]->intListValue();
}


/*** Return value of real parameter "name" ***/

double realParameter(const string& name, int pos=-1)
{ 
  double n = 0;
  
  Parameter* p = (*gParameters)[name];
  if(p) {
    if (p->size() < 0 && pos > -1) {
      errstr << "Parameter '"
	     << name << "' is incorrectly accessed as a list"
	     << Sas::error("ParamNotList");      
    }
    else if(pos < 0)
      n = p->realValue();
    else if(pos < p->size())
      n = p->realListValue()[pos];
    else
      errstr << "Index " << pos << " is out of range "
	     << "[" << 0 << ',' << p->size() << ") for list parameter '" 
	     << name << "'"
	     << Sas::error("ParamIndexRange");
  }
  return n;
}

vector<double> realListParameter(const string& name)
{
    return (*gParameters)[name]->realListValue();
}


/*** Return value of string parameter "name" ***/

string stringParameter(const string& name, int pos=-1)
{ 
  string s;
  
  Parameter* p = (*gParameters)[name];
  if(p) {
    if (p->size() < 0 && pos > -1) {
      errstr << "Parameter '"
	     << name << "' is incorrectly accessed as a list"
	     << Sas::error("ParamNotList");      
    }
    else if(pos < 0)
      s = p->stringValue();
    else if(pos < p->size())
      s = p->stringListValue()[pos];
    else
      errstr << "Index " << pos << " is out of range "
	     << "[" << 0 << ',' << p->size() << ") for list parameter '" 
	     << name << "'"
	     << Sas::error("ParamIndexRange");
  }
  return s;
}

vector<string> stringListParameter(const string& name)
{
  return (*gParameters)[name]->stringListValue();
}


//================================================================
//  Used only by taskmain
//================================================================

/*** Open the parameter set for task "name" ***/

void openParameters(const string& name)
{
  gParameters = ParamInfoSet::instance()->getParamInfo(name);

  if(gParameters == 0)
    errstr << "Invalid parameter file" << Sas::error("ParamInvalidFile");
}


/*** Close the parameter set ***/

void closeParameters()
{
  if (! gParameters) {
    errstr << "Unloading of parameters requested, but no parameters loaded"
	   << Sas::message(Msg::Lib, Msg::Noisy);
    return;
  }
  
  ParamInfoSet::instance()->resetParamInfo(gParameters->taskName());
  
  gParameters = 0;
}
